/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive;

import org.xmloperator.reactive.exception.ReactiveException;

/**
 * An Action impacts the environment of a community.
 * This is the reason while it is always executed during the inactive part
 *   of an instant.
 * There is no deterministic reverse to a given Action.
 */
public interface Action {

	/**
	 * Returns the priority of this Action.
	 * Use the PRIORITY_* constants.
	 * @return the priority of this Action.
	 */
	public int getPriority();
	public static final int PRIORITY_LOW = 1; // Execution at last
	public static final int PRIORITY_MEDIUM = 2; // Default priority
	public static final int PRIORITY_HIGH = 3; // Execution at first

	/**
	 * Returns the exclusive Event, if any, of the Agent to inform about Action
	 * failure.
	 * 
	 * @return the exclusive Event, if any, of the Agent to inform about Action
	 *         failure.
	 */
	public Event getReactionEvent();
	
	/**
	 * Returns the current status of this Action.
	 * @return the current status of this Action.
	 */
	public boolean isExecuted();
	
	/**
	 * Executes this Action. It cannot be executed twice.
	 * @param permissionKey a key that allows this Action.
	 * @exception ReactiveException is thrown if the Action fails.
	 */	
	public void execute(long permissionKey) throws ReactiveException;

	/**
	 * Returns <tt>true</tt> if this Action can be traced, <tt>false</tt>
	 * elsewhere.
	 * 
	 * @return <tt>true</tt> if this Action can be traced, <tt>false</tt>
	 *         elsewhere.
	 */
	public boolean isTraceable();
}
