/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive;

/**
 * An ActivityObserver is used for storying a measurement of the activity
 *	of all the agent and communities of a ReactiveMachine.
 */
public interface ActivityObserver {

	/**
	 * Notify the starting of the given Agent.
	 * @param agent that starts.
	 */
	public void notifyAgentStarting(Agent agent);
	
	/**
	 * Notify the terminating of the given Agent.
	 * @param agent that terminates.
	 */
	public void notifyAgentTerminating(Agent agent);
	
	/**
	 * Clear the activity estimation of all agents.
	 */
	public void clearAgentActivities();

	/**
	 * Returns the list of Agent relative activities.
	 * Activity sum is lesser than 1.0.
	 * Agents are sorted by starting time.
	 * @return the list of Agent relative activities.
	 */
	public AgentActivity[] agentActivities();
}
