/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive;

import org.xmloperator.reactive.exception.SenderUncertifiableException;

/**
 * A Message is attached to a MessageEvent.
 * It is sender certified by the ReactiveEntity.
 */
public interface Message {

	/**
	 * @return <tt>true</tt> if this Message is a SimpleMessage, <tt>false</tt>
	 *         elsewhere.
	 */
	public boolean isSimpleMessage();
	
	/**
	 * @return <tt>true</tt> if this Message is a DoubleMessage, <tt>false</tt>
	 *         elsewhere.
	 */
	public boolean isDoubleMessage();
	
	/**
	 * Returns <tt>true</tt> if this Message is an ActionMessage, <tt>false</tt>
	 * elsewhere.
	 * 
	 * @return <tt>true</tt> if this Message is an ActionMessage, <tt>false</tt>
	 *         elsewhere.
	 */
	public boolean isActionMessage();

	/**
	 * Returns <tt>true</tt> if this Message can be traced, <tt>false</tt>
	 * elsewhere.
	 * 
	 * @return <tt>true</tt> if this Message can be traced, <tt>false</tt>
	 *         elsewhere.
	 */
	public boolean isTraceable();

	/**
	 * Returns the entityId of the sender of this Message. It is set by
	 * the ReactiveEntity used for generating the MessageEvent.
	 * 
	 * @return the entityId of the sender of this Message.
	 */
	public String getSenderEntityId();

	/**
	 * Set the entityId of the sender of this Message. This method is
	 * called by the ReactiveEntity, not directly by the Agent.
	 * 
	 * @param senderEntityId
	 *            the entityId of the sender of this Message.
	 * @throws SenderUncertifiableException
	 *             if the entityId of the sender has already been set by the
	 *             agent. It must not.
	 */
	public void setSenderEntityId(String senderEntityId)
			throws SenderUncertifiableException;

	/**
	 * Returns the destination Cell number if this Message is destined to a
	 * CellArray, -1 elsewhere.
	 * 
	 * @return the destination Cell number if this Message is destined to a
	 *         CellArray, -1 elsewhere.
	 */
	public int getDestinationCellNumber();
}
