/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo;

import org.xmloperator.reactive.Agent;
import org.xmloperator.reactive.CrashingReport;
import org.xmloperator.reactive.Event;
import org.xmloperator.reactive.PrintStream;
import org.xmloperator.reactive.ReactiveCommunity;
import org.xmloperator.reactive.agent.AgentImpl;
import org.xmloperator.reactive.agent.ReactiveFactoryImpl;

import org.xmloperator.reactive.demo.ball.BallPlayerAgent1;
import org.xmloperator.reactive.demo.ball.BallPlayerAgent2;
import org.xmloperator.reactive.demo.ball.BallPlayerCellArray;
import org.xmloperator.reactive.demo.battle.BattleGameEvents;
import org.xmloperator.reactive.demo.battle.BattlePlayerAgent;
import org.xmloperator.reactive.demo.event.EventListenerAgent;
import org.xmloperator.reactive.demo.land.agent.LandDemo;
import org.xmloperator.reactive.demo.land.navigation.DiscoveryNavigationStrategy;
import org.xmloperator.reactive.demo.market.data.MarketDemo;
import org.xmloperator.reactive.demo.market.impl.MarketBoardImpl;
import org.xmloperator.reactive.demo.syracuse.SyracuseAgent;

import org.xmloperator.reactive.impl.PrintStreamImpl;

/**
 * Test of the ReactiveMachine.
 */
public abstract class Test {
	public static void main(String[] args) {
		PrintStream trace = new PrintStreamImpl();
		CrashingReport crashingReport = new CrashingReportImpl(
				trace.getOriginalPrintStream());
		long t1 = System.currentTimeMillis();
		allTests(trace, crashingReport);
		long t2 = System.currentTimeMillis();
		if (crashingReport.isAllOk()) {
			trace.println("No agent crashes");
			trace.println("Uses " + (t2 - t1) + " ms");
		}
		else
			trace.println("Some agent has crashed");
		System.exit(0);
	}
	
	private static void allTests(PrintStream trace, CrashingReport report) {
		String selection = null;
		if (selection == null || selection.equals("stop_suspend"))
			test_1_stop_and_2_suspends(trace, report);
		if (selection == null || selection.equals("syracuse"))
			test_Syracuse(trace, report);
		if (selection == null || selection.equals("event_order"))
			test_eventOrder(trace, report);
		if (selection == null || selection.equals("battle"))
			test_battle(trace, report);
		if (selection == null || selection.equals("pass_ball1"))
			test_passBall1(trace, report);
		if (selection == null || selection.equals("pass_ball2"))
			test_passBall2(trace, report);
		if (selection == null || selection.equals("pass_ball3"))
			test_passBall3(trace, report);
		if (selection == null || selection.equals("market"))
			test_market(trace, report);
		if (selection == null || selection.equals("land"))
			test_land(trace, report);
	}
	
	private static void test_1_stop_and_2_suspends(PrintStream trace,
			CrashingReport crashingReport) {
		ReactiveFactoryImpl.setTrace(trace, "Crtma");
		trace.println("--- test_1_stop_and_2_suspends");
		AgentImpl agent = new AgentImpl(false) {
			public void participate() {
				if (this.entity.getCommunity().getInstant() != 0)
					throw new IllegalStateException();
				this.entity.stop();
				if (this.entity.getCommunity().getInstant() != 1)
					throw new IllegalStateException();
				this.entity.suspend();
				if (this.entity.getCommunity().getInstant() != 2)
					throw new IllegalStateException();
				this.entity.suspend();
				if (this.entity.getCommunity().getInstant() != 3)
					throw new IllegalStateException();
			}
		};
		TestUtil.test(agent, crashingReport);
	}

	private static void test_Syracuse(PrintStream trace,
			CrashingReport crashingReport) {
		ReactiveFactoryImpl.setTrace(trace, "ma");
		trace.println("--- test_Syracuse");
		AgentImpl agent = new SyracuseAgent(1, 5, null);
		TestUtil.testWithExitBoard(agent, crashingReport);
	}

	private static void test_eventOrder(PrintStream trace,
			CrashingReport crashingReport) {
		ReactiveFactoryImpl.setTrace(trace, "a");
		trace.println("--- test_eventOrder");
		AgentImpl agent = new EventListenerAgent(3, 60);
		TestUtil.test(agent, crashingReport);
	}

	private static void test_battle(PrintStream trace,
			CrashingReport crashingReport) {
		ReactiveFactoryImpl.setTrace(trace, "a");
		trace.println("--- test_battle");
		AgentImpl launcher = new AgentImpl(false) {
			public void participate() {
				ReactiveCommunity community = this.entity.getCommunity();
				BattleGameEvents gameEvents = new BattleGameEvents(community,
						4, 8);
				for (int k = 0; k < gameEvents.playerCount; k++) {
					Agent player = new BattlePlayerAgent(k, gameEvents, 3);
					community.launchAgent(player);
				}
			}
		};
		TestUtil.test(launcher, crashingReport);
	}

	private static void test_passBall1(PrintStream trace,
			CrashingReport crashingReport) {
		ReactiveFactoryImpl.setTrace(trace, "a");
		trace.println("--- test_passBall1");
		AgentImpl launcher = new AgentImpl(false) {
			public void participate() {
				ReactiveCommunity community = this.entity.getCommunity();
				Event declarationMessageEvent = community.createMessageEvent();
				Event passBallMessageEvent = community.createMessageEvent();
				int playerCount = 3;
				int stepCount = 3;
				for (int k = 0; k < playerCount; k++) {
					Agent player = new BallPlayerAgent1(playerCount, k,
							stepCount, declarationMessageEvent,
							passBallMessageEvent);
					community.launchAgent(player);
				}
			}
		};
		TestUtil.test(launcher, crashingReport);
	}

	private static void test_passBall2(PrintStream trace,
			CrashingReport crashingReport) {
		ReactiveFactoryImpl.setTrace(trace, "a");
		trace.println("--- test_passBall2");
		AgentImpl launcher = new AgentImpl(false) {
			public void participate() {
				ReactiveCommunity community = this.entity.getCommunity();
				Event declarationMessageEvent = community.createMessageEvent();
				int playerCount = 3;
				int stepCount = 3;
				for (int k = 0; k < playerCount; k++) {
					Agent player = new BallPlayerAgent2(playerCount, k,
							stepCount, declarationMessageEvent);
					community.launchAgent(player);
				}
			}
		};
		TestUtil.test(launcher, crashingReport);
	}

	private static void test_passBall3(PrintStream trace,
			CrashingReport crashingReport) {
		ReactiveFactoryImpl.setTrace(trace, "a");
		trace.println("--- test_passBall3");
		TestUtil.test(new BallPlayerCellArray(3, 3), crashingReport);
	}

	private static void test_market(PrintStream trace,
			CrashingReport crashingReport) {
		ReactiveFactoryImpl.setTrace(trace, "cta");
		MarketBoardImpl.setTrace(trace, "");
		trace.println("--- test_market");
		TestUtil.test(new MarketDemo(), crashingReport);
	}

	private static void test_land(PrintStream trace,
			CrashingReport crashingReport) {
		ReactiveFactoryImpl.setTrace(trace, "a");
//		org.xmloperator.reactive.demo.land.sphere.impl.StationImpl.setTrace(trace);
//		StationsImpl.setTrace(trace);
//		GraphImpl.setTrace(trace);
		DiscoveryNavigationStrategy.setTrace(trace);
		trace.println("--- test_land");
		TestUtil.test(new LandDemo(), crashingReport);
	}
}
