/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.ball;

import org.xmloperator.reactive.Event;
import org.xmloperator.reactive.LaunchingContext;
import org.xmloperator.reactive.Message;
import org.xmloperator.reactive.agent.CellArray;
import org.xmloperator.reactive.impl.SimpleMessageImpl;

/**
 * Array of BallPlayerCell.
 * This is a demonstration of CellArray.
 */
public final class BallPlayerCellArray extends CellArray {
	final int playerCount;
	final int stepCount;
	
	public BallPlayerCellArray(int playerCount, int stepCount) {
		super(playerCount);
		if (playerCount < 2 || stepCount < 0)
			throw new IllegalArgumentException();
		this.playerCount = playerCount;
		this.stepCount = stepCount;
		for (int k = 0; k < playerCount; k++)
			this.setCell(k, new BallPlayerCell(this));
	}
	
	final void passBall(int fromPlayerNumber, int toPlayerNumber) {
		Event event = this.entity.getExclusiveMessageEvent();
		Message message = new SimpleMessageImpl(toPlayerNumber, new Integer(
						fromPlayerNumber));
		this.entity.generateMessage(event, message);
		if (this.trace != null) {
			long instant = this.entity.getCommunity().getInstant();
			this.trace.println(instant + ": " + fromPlayerNumber + ">"
					+ toPlayerNumber);
		}
	}
	
	///////////////////////////////
	// ReactiveAgent implementation
	///////////////////////////////

	public boolean newInstant() {
		long instant = this.entity.getCommunity().getInstant();
		return instant <= this.stepCount;
	}
	
	///////////////////////
	// Agent implementation
	///////////////////////
	
	/**
	 * The player 0 starts by passing a ball forward and another ball backward.
	 */
	public void init(LaunchingContext launchingContext) {
		super.init(launchingContext);
		this.entity.stop();
		this.passBall(0, 1);
		this.passBall(0, this.playerCount - 1);
	}
}
