/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.battle;

import java.util.HashMap;
import java.util.Map;

import org.xmloperator.reactive.Event;
import org.xmloperator.reactive.ReactiveCommunity;

/**
 * Events for BattlePlayerAgent.
 */
public final class BattleGameEvents {
	public final int playerCount;
	public final int cardCountByPlayer;
	
	private final Event[][] cardEvents;
	private final Map<Event, Integer> fromEventToValue = new HashMap<Event, Integer>();
	
	public BattleGameEvents(ReactiveCommunity reactiveCommunity,
			int playerCount, int cardCountByPlayer) {
		if (playerCount < 1 || cardCountByPlayer < 1)
			throw new IllegalArgumentException();
		this.playerCount = playerCount;
		this.cardCountByPlayer = cardCountByPlayer;
		this.cardEvents = new Event[playerCount][cardCountByPlayer];
		for (int player = 0; player < playerCount; player++) {
			Event[] playerEvents = this.cardEvents[player];
			for (int card = 0; card < cardCountByPlayer; card++) {
				Event e = reactiveCommunity.createEvent();
				playerEvents[card] = e;
				this.fromEventToValue.put(e, new Integer(card));
			}
		}
	}
	
	public Event getCardEvent(int player, int card) {
		return this.cardEvents[player][card];
	}
	
	public int getCardValue(Event e) {
		return this.fromEventToValue.get(e).intValue();
	}
}
