/*
 * Copyright (c) 2008, 2011 The_xmloperator_project. All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.land;

import org.xmloperator.reactive.Event;
import org.xmloperator.reactive.exception.IncorrectKeyException;

/**
 * An AgentBody is the visible aspect of an Agent. It is known from the other
 * ones.
 */
public interface AgentBody {

	/**
	 * @return the entity Id of the considered Agent.
	 */
	public String getEntityId();

	/**
	 * @return the agent id as a chain of descendants.
	 */
	public String getAgentId();
	
	/**
	 * @return the Message Event exclusively attached to this Occupant.
	 */
	public Event getExclusiveMessageEvent();
	
	/**
	 * @return the AgentFamily of the considered Agent.
	 */
	public AgentFamily getAgentFamily();

	/**
	 * Returns the current energy of the considered Agent. This request is only
	 * possible from an Agent of the same family, which knows the key attached
	 * to the family.
	 * 
	 * @param familyKey
	 *            the key associated to the AgentFamily.
	 * @return the current energy of the considered Agent.
	 * @throws IncorrectKeyException
	 *             if the given key is not the good one.
	 */
	public int getCurrentEnergy(long familyKey) throws IncorrectKeyException;
}
