/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.land;

/**
 * Graph of paths between stations.
 */
public interface Graph {

	/**
	 * @return the count of stations.
	 */
	public int getStationCount();
	
	/**
	 * @return the minimum path count from any station.
	 */
	public int getMinPathCount();
	
	/**
	 * @return the maximum path count from any station.
	 */
	public int getMaxPathCount();

	/**
	 * @param stationIndex
	 *            the index (0..n-1) of a station.
	 * @return the path count from the given station.
	 */
	public int getPathCount(int stationIndex);

	/**
	 * @param stationIndex
	 *            the index (0..n-1) of a station.
	 * @param pathIndex
	 *            the index (0..pathCount-1) of a path from the given station.
	 * @return the index of the destination station.
	 */
	public int getDestinationIndex(int stationIndex, int pathIndex);

	/**
	 * Get the set of stations accessible from a given station.
	 * 
	 * @param stationIndex
	 *            the index (0..n-1) of a station.
	 * @param destinationIndexes
	 *            a <tt>int[maxPathCount]</tt> that receives the indexes of the
	 *            possible destination stations.
	 */
	public void getDestinationIndexes(int stationIndex, int[] destinationIndexes);
}
