/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.land;

import java.util.List;

/**
 * A Station is occupied by mobile agents.
 * A Station has a source that supplies occupants.
 */
public interface Station {

	/**
	 * @return the index of this Station.
	 */
	public int getStationIndex();
	
	/**
	 * @return the whole stations.
	 */
	public Stations getStations();
	
	/**
	 * @param station
	 *            an other Station.
	 * @return true if the given Station is connected to this one.
	 */
	public boolean isConnectedTo(Station station);

	/**
	 * @return a list of the occupants of this Station.
	 */
	public List<AgentBody> getOccupantList();

	/**
	 * @return the Navigation strategy that consists to going towards this
	 *         Station.
	 */
	public NavigationStrategy getTowardsNavigationStrategy();

	/**
	 * @return the Navigation strategy that consists to going away this Station.
	 */
	public NavigationStrategy getAwayNavigationStrategy();
	
	/**
	 * @return a number of resource units, to be shared each instant by occupants.
	 */
	public int getFlowSource();
}
