/*
 * Copyright (c) 2008, 2011 The_xmloperator_project. All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.land.agent;

import java.util.Random;

import org.xmloperator.reactive.agent.ReactiveFactoryImpl;
import org.xmloperator.reactive.demo.land.AgentFamily;
import org.xmloperator.reactive.demo.land.Graph;
import org.xmloperator.reactive.demo.land.NavigationStrategy;
import org.xmloperator.reactive.demo.land.Stations;
import org.xmloperator.reactive.demo.land.impl.AgentFamilyImpl;
import org.xmloperator.reactive.demo.land.navigation.BiNavigationStrategy;
import org.xmloperator.reactive.demo.land.navigation.DiscoveryNavigationStrategy;
import org.xmloperator.reactive.demo.land.navigation.OccupationNavigationStrategy;
import org.xmloperator.reactive.demo.land.navigation.RandomNavigationStrategy;
import org.xmloperator.reactive.demo.land.navigation.SourceNavigationStrategy;
import org.xmloperator.reactive.demo.land.navigation.SupplyNavigationStrategy;

/**
 * List of Agent families and associated navigation strategies.
 */
final class Families {
	final NavigationStrategy[] navigationStrategies;
	final AgentFamily[] agentFamilies;
	final long[] agentFamilyKeys;
	
	Families(Stations stations, Random random) {
		Graph graph = stations.getGraph();
		this.navigationStrategies = new NavigationStrategy[] {
				new RandomNavigationStrategy(graph, random.nextLong()),
				new DiscoveryNavigationStrategy(graph),
				new OccupationNavigationStrategy(stations, false, false),
				new OccupationNavigationStrategy(stations, true, false),
				new SourceNavigationStrategy(stations, false),
				new SupplyNavigationStrategy(stations, false),
				new BiNavigationStrategy(new OccupationNavigationStrategy(
						stations, false, true), new RandomNavigationStrategy(
						graph, random.nextLong())),
				new BiNavigationStrategy(new OccupationNavigationStrategy(
						stations, true, true), new RandomNavigationStrategy(
						graph, random.nextLong())),
				new BiNavigationStrategy(new SourceNavigationStrategy(stations,
						true), new RandomNavigationStrategy(graph,
						random.nextLong())),
				new BiNavigationStrategy(new SupplyNavigationStrategy(stations,
						true), new RandomNavigationStrategy(graph,
						random.nextLong())) };
		this.agentFamilies = new AgentFamily[this.navigationStrategies.length];
		this.agentFamilyKeys = new long[this.navigationStrategies.length];
		for (int k = 0; k < this.agentFamilies.length; k++) {
			this.agentFamilies[k] = new AgentFamilyImpl("F" + (1 + k));
			this.agentFamilyKeys[k] = ReactiveFactoryImpl.REACTIVE_FACTORY.newKey();
		}
	}
	
	int getFamilyCount() {
		return this.navigationStrategies.length;
	}
	
	NavigationStrategy getNavigationStrategy(int index) {
		return this.navigationStrategies[index];
	}
	
	AgentFamily getAgentFamily(int index) {
		return this.agentFamilies[index];
	}
	
	long getAgentFamilyKey(int index) {
		return this.agentFamilyKeys[index];
	}
	
	////////////////////////
	// Object implementation
	////////////////////////
	
	public String toString() {
		StringBuffer sb = new StringBuffer();
		for (int k = 0; k < this.navigationStrategies.length; k++) {
			if (k != 0)
				sb.append(" ");
			sb.append(this.agentFamilies[k]);
			sb.append(":");
			sb.append(this.navigationStrategies[k]);
		}
		return sb.toString();
	}
}
