/*
 * Copyright (c) 2008, 2011 The_xmloperator_project. All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.land.agent;

import java.util.concurrent.atomic.AtomicInteger;

import org.xmloperator.reactive.demo.land.NavigationStrategy;

/**
 * Statistic on agent strategies.
 */
final class StrategyStat {
	private final Families families;
	private AtomicInteger agentCount = new AtomicInteger();;
	private final AtomicInteger[] navigationStrategyCounts;
	
	StrategyStat(Families families) {
		if (families == null)
			throw new IllegalArgumentException();
		this.families = families;
		int familyCount = families.getFamilyCount();
		this.navigationStrategyCounts = new AtomicInteger[familyCount];
		for (int k = 0; k < familyCount; k++)
			this.navigationStrategyCounts[k] = new AtomicInteger();
	}
	
	void add(GenePool genePool) {
		this.agentCount.incrementAndGet();
		NavigationStrategy navigationStrategy = genePool.getNavigationStrategy();
		int familyCount = this.families.getFamilyCount();
		for (int k = 0; k < familyCount; k++)
			if (this.families.getNavigationStrategy(k) == navigationStrategy) {
				this.navigationStrategyCounts[k].incrementAndGet();
				return;
			}
		String navigationStrategyName = navigationStrategy.toString();
		for (int k = 0; k < familyCount; k++)
			if (this.families.getNavigationStrategy(k).toString()
					.equals(navigationStrategyName))
				this.navigationStrategyCounts[k].incrementAndGet();
	}
	
	////////////////////////
	// Object implementation
	////////////////////////
	
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append("Total(");
		sb.append(this.agentCount);
		sb.append(")");
		int familyCount = this.families.getFamilyCount();
		for (int k = 0; k < familyCount; k++) {
			sb.append(" ");
			sb.append(this.families.getNavigationStrategy(k));
			sb.append("(");
			sb.append(this.navigationStrategyCounts[k]);
			sb.append(")");
		}
		return sb.toString();
	}
}
