/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.land.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.xmloperator.reactive.demo.land.AgentBody;
import org.xmloperator.reactive.demo.land.AgentFamily;

/**
 * AgentFamily implementation.
 */
public class AgentFamilyImpl implements AgentFamily {
	private final String familyId;
	private final List<AgentBody> memberList = new ArrayList<AgentBody>();
	private final List<AgentBody> unmodifiableMemberList = Collections
			.unmodifiableList(this.memberList);

	public AgentFamilyImpl(String familyId) {
		if (familyId == null)
			throw new IllegalArgumentException();
		this.familyId = familyId;
	}
	
	/**
	 * Adds a member.
	 * @param agentBody a new member.
	 */
	final void addMember(AgentBody agentBody) {
		if (this.memberList.contains(agentBody))
			throw new IllegalStateException();
		this.memberList.add(agentBody);
	}
	
	/**
	 * Removes a member.
	 * @param agentBody a terminated member.
	 */
	final void removeMember(AgentBody agentBody) {
		if (!this.memberList.contains(agentBody))
			throw new IllegalStateException();
		this.memberList.remove(agentBody);
	}
	
	/////////////////////////////
	// AgentFamily implementation
	/////////////////////////////
	
	public final String getFamilyId() {
		return this.familyId;
	}
	
	public final List<AgentBody> getMemberList() {
		return this.unmodifiableMemberList;
	}
	
	////////////////////////
	// Object implementation
	////////////////////////

	public String toString() {
		return this.familyId;
	}
}
