/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.land.impl;

import java.util.Random;

import org.xmloperator.reactive.demo.land.AgentBody;
import org.xmloperator.reactive.demo.land.Graph;
import org.xmloperator.reactive.demo.land.Station;
import org.xmloperator.reactive.demo.land.Stations;

/**
 * Stations implementation.
 */
public class StationsImpl implements Stations {
	final Graph graph;
	private final StationImpl[] stations;
	private int occupantCount = 0;
	
	public StationsImpl(Graph graph, Random random) {
		this.graph = graph;
		int stationCount = graph.getStationCount();
		this.stations = new StationImpl[stationCount];
		for (int k = 0; k < stationCount; k++) {
			int flowSource = random.nextInt(5) * random.nextInt(5);
			this.stations[k] = new StationImpl(this, k, flowSource);
		}
	}
	
	/**
	 * An Agent moves from a Station to another Station.
	 * 
	 * @param agentBody
	 *            an AgentBody.
	 * @param fromStationIndex
	 *            the index of the starting station or -1 if none.
	 * @param toStationIndex
	 *            the index of the destination station or -1 if none.
	 */
	void move(AgentBody agentBody, int fromStationIndex, int toStationIndex) {
		if (fromStationIndex >= 0) {
			this.stations[fromStationIndex].removeOccupant(agentBody);
			this.occupantCount--;
		}
		if (toStationIndex >= 0) {
			this.stations[toStationIndex].addOccupant(agentBody);
			this.occupantCount++;
		}
	}

	//////////////////////////
	// Stations implementation
	//////////////////////////
	
	public final Graph getGraph() {
		return this.graph;
	}

	public Station getStation(int stationIndex) {
		return this.stations[stationIndex];
	}
	
	public final Station anyStation(Random random) {
		int stationIndex = random.nextInt(this.graph.getStationCount());
		return this.stations[stationIndex];
	}

	public final int getOccupantCount() {
		return this.occupantCount;
	}
	
	////////////////////////
	// Object implementation
	////////////////////////
	
	public String toString() {
		StringBuffer sb = new StringBuffer();
		for (int k = 0; k < this.stations.length; k++) {
			StationImpl station = this.stations[k];
			if (this.occupantCount == 0
					|| !station.getOccupantList().isEmpty()) {
				if (k != 0)
					sb.append(" ");
				sb.append(station);
			}
		}
		return sb.toString();
	}
}
