/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.land.impl;

import java.util.Arrays;

import org.xmloperator.reactive.demo.land.VisitMemory;

/**
 * VisitMemory implementation.
 */
public final class VisitMemoryImpl implements VisitMemory {
	private int lastMoveNumber = 0;
	private int[] stationsLastVisit;
	private int lastVisitInterval = 0;

	/**
	 * Constructor
	 * 
	 * @param stationCount
	 *            the number of stations in the Graph.
	 */
	public VisitMemoryImpl(int stationCount) {
		if (stationCount < 1)
			throw new IllegalArgumentException();
		this.stationsLastVisit = new int[stationCount];
		Arrays.fill(this.stationsLastVisit, 0);
	}

	/////////////////////////////
	// VisitMemory implementation
	/////////////////////////////

	public void visit(int stationIndex) {
		this.lastMoveNumber++;
		this.lastVisitInterval = this.lastMoveNumber
				- this.stationsLastVisit[stationIndex];
		this.stationsLastVisit[stationIndex] = this.lastMoveNumber;
	}

	public int getLastVisit(int stationIndex) {
		return this.stationsLastVisit[stationIndex];
	}

	public int getLastMoveNumber() {
		return this.lastMoveNumber;
	}

	public int getLastVisitInterval() {
		return this.lastVisitInterval;
	}
	
	////////////////////////
	// Object implementation
	////////////////////////
	
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append("[");
		for (int k = 0; k < this.stationsLastVisit.length; k++)
			sb.append(this.stationsLastVisit[k] == 0 ? "-" : "+");
		sb.append("]");
		return sb.toString();
	}
}
