/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.land.navigation;

import java.util.Random;

import org.xmloperator.reactive.demo.land.Graph;
import org.xmloperator.reactive.demo.land.NavigationStrategy;
import org.xmloperator.reactive.demo.land.VisitMemory;

public final class RandomNavigationStrategy extends NavigationStrategyImpl {
	private final Graph graph;
	private final Random random;
	
	public RandomNavigationStrategy(Graph graph, long seed) {
		if (graph == null)
			throw new IllegalArgumentException();
		this.graph = graph;
		this.random = new Random(seed);
	}
	
	////////////////////////////////////
	// NavigationStrategy implementation
	////////////////////////////////////
	
	public final int nextStationIndex(int stationIndex, VisitMemory visitMemory) {
		int pathCount = this.graph.getPathCount(stationIndex);
		int pathIndex = this.random.nextInt(pathCount);
		int newStationIndex = this.graph.getDestinationIndex(stationIndex,
				pathIndex);
		if (visitMemory != null)
			visitMemory.visit(newStationIndex);
		return newStationIndex;
	}

	public NavigationStrategy almostClone(long seed) {
		return new RandomNavigationStrategy(this.graph, seed);
	}

	////////////////////////
	// Object implementation
	////////////////////////
	
	public String toString() {
		return "random";
	}
}
