/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.land.navigation;

import org.xmloperator.reactive.demo.land.Graph;
import org.xmloperator.reactive.demo.land.Station;
import org.xmloperator.reactive.demo.land.Stations;
import org.xmloperator.reactive.demo.land.VisitMemory;

/**
 * Navigation strategy that consists in going to the station with the greater
 * supply (== flowSource / occupantCount).
 */
public final class SupplyNavigationStrategy extends NavigationStrategyImpl {
	private Stations stations;
	private final boolean isCurrentStationIncluded;
	private final int[] destinationIndexes;

	public SupplyNavigationStrategy(Stations stations,
			boolean isCurrentStationIncluded) {
		if (stations == null)
			throw new IllegalArgumentException();
		this.stations = stations;
		this.isCurrentStationIncluded = isCurrentStationIncluded;
		this.destinationIndexes = new int[stations.getGraph().getMaxPathCount()];
	}

	////////////////////////////////////
	// NavigationStrategy implementation
	////////////////////////////////////
	
	public int nextStationIndex(int stationIndex, VisitMemory visitMemory) {
		Station station = this.stations.getStation(stationIndex);
		int greaterSupply = this.isCurrentStationIncluded ? station
				.getFlowSource() / station.getOccupantList().size() : -1;
		int greaterSupplyIndex = stationIndex;
		Graph graph = this.stations.getGraph();
		int pathCount = graph.getPathCount(stationIndex);
		synchronized (this) {
			graph.getDestinationIndexes(stationIndex, this.destinationIndexes);
			for (int k = 0; k < pathCount; k++) {
				int destinationIndex = this.destinationIndexes[k];
				Station destination = this.stations.getStation(destinationIndex);
				int supply = destination.getFlowSource()
						/ (destination.getOccupantList().size() + 1);
				if (supply > greaterSupply) {
					greaterSupply = supply;
					greaterSupplyIndex = destinationIndex;
				}
			}
		}
		if (visitMemory != null)
			visitMemory.visit(greaterSupplyIndex);
		return greaterSupplyIndex;
	}

	////////////////////////
	// Object implementation
	////////////////////////
	
	public String toString() {
		return "supply";
	}
}
