/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.land.navigation;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.xmloperator.reactive.demo.land.Graph;
import org.xmloperator.reactive.demo.land.NavigationStrategy;
import org.xmloperator.reactive.demo.land.Station;
import org.xmloperator.reactive.demo.land.Stations;
import org.xmloperator.reactive.demo.land.VisitMemory;

/**
 * Navigation strategy that consists to go towards, or away a given Station.
 */
public class TargetNavigationStrategies {
	
	private class TargetNavigationStrategy extends NavigationStrategyImpl {
		final boolean isTowards;
		
		TargetNavigationStrategy(boolean isTowards) {
			this.isTowards = isTowards;
		}

		////////////////////////////////////
		// NavigationStrategy implementation
		////////////////////////////////////
		
		public final int nextStationIndex(int stationIndex,
				VisitMemory visitMemory) {
			int pathCount = graph.getPathCount(stationIndex);
			int bestDistance = distances[stationIndex];
			int bestDestinationIndex = stationIndex;
			synchronized (this) {
				graph.getDestinationIndexes(stationIndex,
						destinationIndexes);
				for (int k = 0; k < pathCount; k++) {
					int destinationIndex = destinationIndexes[k];
					int distance = distances[destinationIndex];
					if (this.isTowards ? (distance < bestDistance)
							: (distance > bestDistance)) {
						bestDistance = distance;
						bestDestinationIndex = destinationIndex;
					}
				}
			}
			if (visitMemory != null)
				visitMemory.visit(bestDestinationIndex);
			return bestDestinationIndex;
		}

		////////////////////////
		// Object implementation
		////////////////////////
		
		public String toString() {
			return "target(" + (this.isTowards ? "to" : "from") + ":" + target
					+ ")";
		}
	}
	
	private final Graph graph;
	private final Station target;
	private final int[] distances;
	private final int[] destinationIndexes;
	private NavigationStrategy towardsTargetNavigationStrategy = null;
	private NavigationStrategy awayTargetNavigationStrategy = null;
	
	public TargetNavigationStrategies(Stations stations, Station target) {
		if (target == null)
			throw new IllegalArgumentException();
		this.graph = stations.getGraph();
		this.target = target;
		this.distances = new int[graph.getStationCount()];
		Arrays.fill(this.distances, -1);
		this.distances[target.getStationIndex()] = 0;
		List<Station> stationsToComplete = new ArrayList<Station>();
		stationsToComplete.add(target);
		this.destinationIndexes = new int[graph.getMaxPathCount()];
		while (!stationsToComplete.isEmpty()) {
			Station station1 = stationsToComplete.remove(0);
			int station1Index = station1.getStationIndex();
			int distance = this.distances[station1Index];
			int pathCount = graph.getPathCount(station1Index);
			this.graph.getDestinationIndexes(station1Index, destinationIndexes);
			for (int k = 0; k < pathCount; k++) {
				int station2Index = destinationIndexes[k];
				if (this.distances[station2Index] < 0) {
					this.distances[station2Index] = distance + 1;
					Station station2 = stations.getStation(station2Index);
					stationsToComplete.add(station2);
				}
			}
		}
	}
	
	public NavigationStrategy getTowardsTargetNavigationStrategy() {
		if (this.towardsTargetNavigationStrategy == null)
			this.towardsTargetNavigationStrategy = new TargetNavigationStrategy(
					true);
		return this.towardsTargetNavigationStrategy;
	}

	public NavigationStrategy getAwayTargetNavigationStrategy() {
		if (this.awayTargetNavigationStrategy == null)
			this.awayTargetNavigationStrategy = new TargetNavigationStrategy(
					false);
		return this.awayTargetNavigationStrategy;
	}
}
