/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.land.sphere.impl;

import org.xmloperator.reactive.demo.land.sphere.Station;
import org.xmloperator.reactive.demo.land.sphere.StationPath;
import org.xmloperator.reactive.demo.land.sphere.Vector2;
import org.xmloperator.reactive.demo.land.sphere.Vector3;

/**
 * StationPath implementation.
 */
public final class StationPathImpl implements StationPath {
	static final double SQRT2 = Math.sqrt(2.0);
	
	/**
	 * @param cosine the cosine of an angle.
	 * @param sine the sine of an angle.
	 * @return the pseudo angle, comprise between -4.0 to 4.0.
	 */
	static final double pseudoAngle(double cosine, double sine) {
		return
			(cosine >= 1.0 / SQRT2) ? SQRT2 * sine :
			(sine >= 1.0 / SQRT2) ? 2.0 - SQRT2 * cosine :
			(sine <= -1.0 / SQRT2) ? -2.0 + SQRT2 * cosine :
			(sine >= 0 ? 4.0 : -4.0) - SQRT2 * sine;
	}
	
//	static {
//	for (int k = 0; k < 40; k++) {
//		double a = k * 2.0 * Math.PI / 40;
//		double cosine = Math.cos(a);
//		double sine = Math.sin(a);
//		System.out.println(k + " "
//				+ Vector2Impl.toString(pseudoAngle(cosine, sine)));
//	}
//	}

	final Station origin;
	final Station destination;
	
	private double pseudoDistance = 0.0;
	private double repulsion = 0.0;
	private Vector2 tangent = null;
	
	public StationPathImpl(Station origin, Station destination) {
		this.origin = origin;
		this.destination = destination;
	}

	final void update() {
		Vector3 p0 = this.origin.getLocation();
		Vector3 p1 = this.destination.getLocation();
		{
			double cosine = Vector3Util.scalarProduct(p0, p1);
			double sine = Vector3Util.vectorProductNorm(p0, p1);
			this.pseudoDistance = StationPathImpl.pseudoAngle(cosine, sine);
			this.repulsion = (1.0 + cosine) / (1.0 + 1e-9 - cosine);
		}
		{
			double u = Vector3Util.scalarProduct(p1, this.origin.getTangentU());
			double v = Vector3Util.scalarProduct(p1, this.origin.getTangentV());
			this.tangent = Vector2Util.normalizedVector(u, v);
		}
	}
	
	/////////////////////////////
	// StationPath implementation
	/////////////////////////////
	
	public final Station getOriginStation() {
		return this.origin;
	}
	
	public final Station getDestinationStation() {
		return this.destination;
	}
	
	public final double getPseudoDistance() {
		return this.pseudoDistance;
	}

	public final double getRepulsion() {
		return this.repulsion;
	}
	
	public Vector2 getTangent() {
		return this.tangent;
	}
	
	////////////////////////
	// Object implementation
	////////////////////////
	
	public String toString() {
		StringBuffer sb = new StringBuffer();
		int id = ((StationImpl)this.destination).id;
		if (id < 10)
			sb.append('0');
		sb.append(id);
		sb.append(":");
		sb.append(Vector2Impl.toString(this.getPseudoDistance()).substring(1));
		return sb.toString();
	}
}
