/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.land.sphere.impl;

import org.xmloperator.reactive.demo.land.sphere.Vector2;

/**
 * Utilities about Vector2.
 */
public abstract class Vector2Util {

	/**
	 * @param v a vector.
	 * @return the norm of the given vector.
	 */
	public static final double norm(Vector2 v) {
		return Vector2Impl.norm(v.getU(), v.getV());
	}
	
	/**
	 * @param u the first coordinate.
	 * @param v the second coordinate.
	 * @return the given vector, normalized.
	 */
	public static final Vector2Impl normalizedVector(double u, double v) {
		double r = Vector2Impl.norm(u, v);
		return new Vector2Impl(u / r, v / r);
	}

	/**
	 * @param v a vector.
	 * @return the given vector, normalized.
	 */
	public static final Vector2Impl normalizedVector(Vector2 v) {
		return normalizedVector(v.getU(), v.getV());
	}

	/**
	 * @param r a scalar.
	 * @param v a vector.
	 * @return the product of the vector by the scalar.
	 */
	public static final Vector2Impl scalarProduct(double r, Vector2 v) {
		return new Vector2Impl(r * v.getU(), r * v.getV());
	}
	
	/**
	 * @param v1 a first vector
	 * @param v2 a second vector
	 * @return the scalar product between the two vectors.
	 */
	public static final double scalarProduct(Vector2 v1, Vector2 v2) {
		return v1.getU() * v2.getU() + v1.getV() * v2.getV();
	}
}
