/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.land.sphere.impl;

import org.xmloperator.reactive.demo.land.sphere.Vector3;

/**
 * Utilities about Vector3.
 */
public abstract class Vector3Util {

	/**
	 * @param v a vector.
	 * @return the norm of the given vector.
	 */
	public static final double norm(Vector3 v) {
		return Vector3Impl.norm(v.getX(), v.getY(), v.getZ());
	}
	
	/**
	 * @param x the first coordinate.
	 * @param y the second coordinate.
	 * @param z the third coordinate.
	 * @return the given vector, normalized.
	 */
	public static final Vector3Impl normalizedVector(double x, double y, double z) {
		double r = Vector3Impl.norm(x, y, z);
		return new Vector3Impl(x / r, y / r, z / r);
	}

	/**
	 * @param v a vector.
	 * @return the given vector, normalized.
	 */
	public static final Vector3Impl normalizedVector(Vector3 v) {
		return normalizedVector(v.getX(), v.getY(), v.getZ());
	}

	/**
	 * @param r a scalar.
	 * @param v a vector.
	 * @return the product of the vector by the scalar.
	 */
	public static final Vector3Impl scalarProduct(double r, Vector3 v) {
		return new Vector3Impl(r * v.getX(), r * v.getY(), r * v.getZ());
	}
	
	/**
	 * @param v1 a first vector
	 * @param v2 a second vector
	 * @return the scalar product between the two vectors.
	 */
	public static final double scalarProduct(Vector3 v1, Vector3 v2) {
		return v1.getX() * v2.getX() + v1.getY() * v2.getY() + v1.getZ() * v2.getZ();
	}
	
	/**
	 * @param v1 a first vector
	 * @param v2 a second vector
	 * @return the norm of the vector product of the two vectors.
	 */
	public static final double vectorProductNorm(Vector3 v1, Vector3 v2) {
		double p1x = v1.getX();
		double p1y = v1.getY();
		double p1z = v1.getZ();
		double p2x = v2.getX();
		double p2y = v2.getY();
		double p2z = v2.getZ();
		double x = p1y * p2z - p1z * p2y;
		double y = p1z * p2x - p1x * p2z;
		double z = p1x * p2y - p1y * p2x;
		return Vector3Impl.norm(x, y, z);
	}

	/**
	 * @param v1 a first vector
	 * @param v2 a second vector
	 * @return the normalized vector product of the two vectors.
	 */
	public static final Vector3Impl normalizedVectorProduct(Vector3 v1, Vector3 v2) {
		double p1x = v1.getX();
		double p1y = v1.getY();
		double p1z = v1.getZ();
		double p2x = v2.getX();
		double p2y = v2.getY();
		double p2z = v2.getZ();
		double x = p1y * p2z - p1z * p2y;
		double y = p1z * p2x - p1x * p2z;
		double z = p1x * p2y - p1y * p2x;
		double r = Vector3Impl.norm(x, y, z);
		return new Vector3Impl(x / r, y / r, z / r);
	}
	
	/**
	 * @param p1 a first point.
	 * @param p2 a second point.
	 * @return the distance between the two point.
	 */
	public static final double distance(Vector3 p1, Vector3 p2) {
		double x = p2.getX() - p1.getX();
		double y = p2.getY() - p1.getY();
		double z = p2.getZ() - p1.getZ();
		return Vector3Impl.norm(x, y, z);
	}
}
