/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.market.action;

import java.math.BigInteger;

import org.xmloperator.reactive.demo.market.MarketBoard;
import org.xmloperator.reactive.demo.market.Resource;
import org.xmloperator.reactive.demo.market.Transformation;
import org.xmloperator.reactive.impl.ActionImpl;

/**
 * Action of executing a Transformation.
 */
public final class TransformAction extends ActionImpl {
	protected static Resource traceableResource;
	
	public static void setTraceableResource(Resource traceableResource) {
		TransformAction.traceableResource = traceableResource;
	}

	public final MarketBoard marketBoard;
	public final Transformation transformation;
	public final BigInteger factor;
	
	public TransformAction(MarketBoard marketBoard,
			Transformation transformation, BigInteger factor) {
		super(PRIORITY_HIGH);
		if (marketBoard == null || transformation == null
				|| factor.compareTo(BigInteger.ONE) < 0)
			throw new IllegalArgumentException();
		this.marketBoard = marketBoard;
		this.transformation = transformation;
		this.factor = factor;
	}
	
	////////////////////////
	// Action implementation
	////////////////////////
	
	public void execute(long permissionKey) {
		super.execute(permissionKey);
		this.marketBoard.newTransform(permissionKey, this.transformation,
				this.factor).execute();
	}
	
	public boolean isTraceable() {
		return traceableResource == null
				|| this.transformation.getIngredient(traceableResource) != null;
	}

	////////////////////////
	// Object implementation
	////////////////////////
	
	public String toString() {
		return "Transform(" + this.transformation + "," + this.factor + ")";
	}
}
