/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.market.agent;

import org.xmloperator.reactive.demo.market.Proposal;

/**
 * A marketing Campaign can be launched by an Operator at each instant. It starts
 * by publishing a Proposal at instant t. At instant t+1 orders are generated.
 * At instant t+2 orders are processed and deliveries are done.
 */
public final class Campaign {
	public final long proposalInstant;
	public final Proposal proposal;
	
	/**
	 * Constructor.
	 * @param proposalInstant the instant of Proposal publishing.
	 * @param proposal a Proposal.
	 */
	public Campaign(long proposalInstant, Proposal proposal) {
		if (proposalInstant < 0 || proposal == null)
			throw new IllegalArgumentException();
		this.proposalInstant = proposalInstant;
		this.proposal = proposal;
	}
}
