/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.market.data;

import java.math.BigInteger;

import org.xmloperator.reactive.demo.market.Resource;
import org.xmloperator.reactive.demo.market.impl.ResourceImpl;

/**
 * An example of MarketData.
 */
public final class ElectricityMarketData implements MarketData {
	public final ResourceImpl euro;
	public final ResourceImpl electricity;

	public final Object[] electricityGeneration;
	public final Object[] electricityDelivery;
	
	public final Object[][] transformers;
	public final Object[][] operators;
	
	public ElectricityMarketData(long initiatorKey) {
		this.euro = new ResourceImpl("Euro",
				initiatorKey, 1.00, 60.0);
		this.electricity = new ResourceImpl("Electricity(kwh)",
				initiatorKey, 0.11, 40.0);
		
		this.electricityGeneration = new Object[] {
			"ElectricityGeneration",
			new Resource[] {euro,	electricity},
			new int[] {		-100,	+1000},
			new Resource[][] {
				{electricity, euro}},
			BigInteger.valueOf(100),
			euro,
			BigInteger.valueOf(8000)
		};
		this.electricityDelivery = new Object[] {
				"ElectricityDelivery",
				new Resource[] {electricity,	euro},
				new int[] {		-1000,		+120},
				new Resource[][] {
					{euro, electricity}},
				BigInteger.valueOf(100),
				euro,
				BigInteger.valueOf(8000)
			};

		this.transformers = new Object[][] {
				electricityGeneration,
				electricityDelivery };
		this.operators = new Object[][] {
			{euro, electricity, BigInteger.valueOf(15000)}};
	}
	
	////////////////////////////
	// MarketData implementation
	////////////////////////////

	public Object[][] getTransformers() {
		return this.transformers;
	}
	
	public Object[][] getOperators() {
		return this.operators;
	}
}
