/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.market.data;

import java.math.BigInteger;

import org.xmloperator.reactive.demo.market.Resource;
import org.xmloperator.reactive.demo.market.impl.ResourceImpl;

/**
 * An example of MarketData.
 */
public final class LemonadeMarketData implements MarketData {
	public final ResourceImpl euro;
	public final ResourceImpl lemonade;
	public final ResourceImpl drinkingWater;
	public final ResourceImpl sugar;
	public final ResourceImpl lemonJuice;

	public final Object[] lemonadeProduction;
	public final Object[] retail;
	public final Object[] waterDistribution;
	public final Object[] sugarProduction;
	public final Object[] lemonJuiceProduction;
	
	public final Object[][] transformers;
	public final Object[][] operators;
	
	public LemonadeMarketData(long initiatorKey) {
		this.euro = new ResourceImpl("Euro",
				initiatorKey, 1.40, 60.0);
		this.lemonade = new ResourceImpl("Lemonade(liter)",
				initiatorKey, 0.70, 5.0);
		this.drinkingWater = new ResourceImpl("DrinkingWater(m3)",
				initiatorKey, 4.2, 20.0);
		this.sugar = new ResourceImpl("Sugar(kg)",
				initiatorKey, 0.70, 10.0);
		this.lemonJuice = new ResourceImpl("LemonJuice(liter)",
				initiatorKey, 6.0, 5.0);
		
		this.lemonadeProduction = new Object[] {
			"LemonadeProduction",
			new Resource[] {drinkingWater, 	sugar,	lemonJuice,	euro,	lemonade},
			new int[] {		-1, 			-100, 	-5,			-400,	+1000},
			new Resource[][] {
				{euro, drinkingWater},
				{euro, sugar},
				{euro, lemonJuice},
				{lemonade, euro}},
			BigInteger.valueOf(10),
			euro,
			BigInteger.valueOf(5000)
		};
		this.retail = new Object[] {
			"Retail",
			new Resource[] {lemonade,	euro},
			new int[] {		-100,		+58},
			new Resource[][] {
				{euro, lemonade}},
			BigInteger.valueOf(20),
			euro,
			BigInteger.valueOf(5000)
		};
		this.waterDistribution = new Object[] {
			"WaterDistribution",
			new Resource[] {euro,	drinkingWater},
			new int[] {		-32,	+11},
			new Resource[][] {
				{drinkingWater, euro}},
			BigInteger.valueOf(1000),
			euro,
			BigInteger.valueOf(3000)
		};
		this.sugarProduction = new Object[] {
			"SugarProduction",
			new Resource[] {euro,	sugar},
			new int[] {		-100,	+210},
			new Resource[][] {
				{sugar, euro}},
			BigInteger.valueOf(100),
			euro,
			BigInteger.valueOf(5000)
		};
		this.lemonJuiceProduction = new Object[] {
			"LemonJuiceProduction",
			new Resource[] {euro,	lemonJuice},
			new int[] {		-100,	+24},
			new Resource[][] {
				{lemonJuice, euro}},
			BigInteger.valueOf(50),
			euro,
			BigInteger.valueOf(5000)
		};

		this.transformers = new Object[][] { lemonadeProduction,
				retail, waterDistribution, sugarProduction,
				lemonJuiceProduction };
		this.operators = new Object[][] {
			{euro, lemonade,		BigInteger.valueOf(5000)},
			{euro, drinkingWater,	BigInteger.valueOf(5000)},
			{euro, sugar,			BigInteger.valueOf(5000)},
			{euro, lemonJuice,		BigInteger.valueOf(5000)}};
	}
	
	////////////////////////////
	// MarketData implementation
	////////////////////////////

	public Object[][] getTransformers() {
		return this.transformers;
	}
	
	public Object[][] getOperators() {
		return this.operators;
	}
}
