/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.market.impl;

import java.math.BigInteger;

import org.xmloperator.reactive.demo.market.Account;
import org.xmloperator.reactive.demo.market.ProjectedAccount;
import org.xmloperator.reactive.demo.market.Bank;
import org.xmloperator.reactive.demo.market.Resource;
import org.xmloperator.reactive.exception.IncorrectKeyException;

final class BankImpl implements Bank {
	private final Resource resource;
	private final long initiatorKey;
	
	BankImpl(Resource resource, long initiatorKey) {
		if (resource == null)
			throw new IllegalArgumentException();
		this.resource = resource;
		this.initiatorKey = initiatorKey;
	}

	//////////////////////
	// Bank implementation
	//////////////////////
	
	public final Resource getResource() {
		return this.resource;
	}

	public final Account createAccount(long holderKey) {
		return new AccountImpl(this, holderKey);
	}

	public final ProjectedAccount createProjectedAccount(long holderKey) {
		return new ProjectedAccountImpl(this, holderKey);
	}

	public final Account createReceivableAccount(long initiatorKey,
			long holderKey, BigInteger maxDebt) throws IncorrectKeyException {
		if (initiatorKey != this.initiatorKey)
			throw new IncorrectKeyException();
		Account account = new ReceivableAccountImpl(this, holderKey, maxDebt);
		return account;
	}

	////////////////////////
	// Object implementation
	////////////////////////
	
	public String toString() {
		return "Bank(" + this.resource + ")";
	}
}
