/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.market.impl;

import org.xmloperator.reactive.demo.market.Ingredient;
import org.xmloperator.reactive.demo.market.Transformation;

/**
 * Utilities about transformations.
 */
public abstract class TransformationUtils {

	/**
	 * Returns the virtual profit value obtained by the execution of the given
	 * Transformation.
	 * @param transformation a Transformation.
	 * @return the virtual profit value obtained by the execution of the given
	 *         Transformation.
	 */
	public static final double getProfit(Transformation transformation) {
		double profit = 0.0;
		for (Ingredient ingredient : transformation.getIngredientList())
			profit += ingredient.getIngredientValue();
		return profit;
	}

	/**
	 * Returns the virtual turnover obtained by the execution of the given
	 * Transformation.
	 * 
	 * @param transformation a Transformation.
	 * @return the virtual turnover obtained by the execution of the given
	 *         Transformation.
	 */
	public static final double getTurnover(Transformation transformation) {
		double turnover = 0.0;
		for (Ingredient ingredient : transformation.getIngredientList())
			turnover += Math.abs(ingredient.getIngredientValue());
		return turnover;
	}

	/**
	 * Returns the virtual relative (to turnover) profit value obtained by the
	 * execution of the given Transformation.
	 * 
	 * @param transformation a Transformation.
	 * @return the virtual relative (to turnover) profit value obtained by the
	 *         execution of the given Transformation.
	 */
	public static final double getRelativeProfit(Transformation transformation) {
		return getProfit(transformation) / getTurnover(transformation);
	}
}
