/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.element;

import org.xmloperator.reactive.element.exception.ChildrenConstraintException;

/**
 * Constraint on the children of a ParentElement.
 */
public interface ChildrenConstraint {
	
	/**
	 * Consider the possibility of adding or removing a given child Element
	 *   to a given ParentElement and throws an Exception
	 *   if this given action doesn't respect this constraint.
	 * @param parentElement a ParentElement.
	 * @param addedOrRemovedChildElement an Element to be added or removed
	 *   as child of the ParentElement.
	 * @param isChildAdded true for adding, false for removing.
	 * @throws ChildrenConstraintException if the adding child operation
	 *   doesn't respect this constraint.
	 */
	public void verifyConstraint(
			ParentElement parentElement,
			Element addedOrRemovedChildElement, boolean isChildAdded)
			throws ChildrenConstraintException;
}
