/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.element;

import java.util.Comparator;

import org.xmloperator.reactive.element.exception.ChildrenConstraintException;

/**
 * Read-write ParentElement.
 */
public interface RWParentElement extends RWElement {

	/**
	 * Append a child to this ParentElement.
	 * @param element an Element to append.
	 * @throws IllegalArgumentException if the given Element has a parent or
	 *   is the root Element of the hierarchy that contains this Element.
	 * @throws ChildrenConstraintException if this operation doesn't respect
	 *   a ChildrenConstraint on this Element.
	 */
	public void appendChild(Element element)
		throws IllegalArgumentException, ChildrenConstraintException;
	
	/**
	 * Removes all the children of this ParentElement.
	 */
	public void removeChildren();
	
	/**
	 * Sorts the children of this ParentElement, using a Comparator.
	 * @param elementComparator a Comparator for Element.
	 */	
	public void sortChildren(Comparator<Element> elementComparator);
}
