/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.element.constraint;

import org.xmloperator.reactive.element.exception.ChildrenConstraintException;

import org.xmloperator.reactive.element.ChildrenConstraint;
import org.xmloperator.reactive.element.Element;
import org.xmloperator.reactive.element.ParentElement;

/**
 * This ChildrenConstraint fixes the name shared by all the children.
 */
public final class FixedNameChildrenConstraint
		implements ChildrenConstraint {
	
	public final String namespaceURI;
	public final String localName;
	
	public FixedNameChildrenConstraint(String namespaceURI, String localName) {
		this.namespaceURI =
			namespaceURI != null
				? namespaceURI.intern()
				: null;
		this.localName = localName;
	}
	
	/////////////////////
	// ChildrenConstraint
	/////////////////////
	
	public void verifyConstraint(
			ParentElement parentElement,
			Element addedOrRemovedChildElement, boolean isChildAdded)
			throws ChildrenConstraintException {
		if (isChildAdded
				&& (addedOrRemovedChildElement.getNamespaceURI()
						!= this.namespaceURI
					|| !addedOrRemovedChildElement.getLocalName().equals(
							this.localName))) {
//			System.err.println(
//				"AddElement(\""
//				+ addedOrRemovedChildElement.getNamespaceURI()
//				+ "\",\""
//				+ addedOrRemovedChildElement.getLocalName()
//				+ "\")");
			throw new ChildrenConstraintException(this);
		}
	}
	
	////////////////////////
	// Object implementation
	////////////////////////

	public String toString() {
		return "F(\"" + this.namespaceURI + "\",\"" + this.localName + "\")";
	}
}
