/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.element.impl;


import org.xmloperator.reactive.element.exception.BadCharInNameException;
import org.xmloperator.reactive.element.exception.IllegalElementChangeException;

import org.xmloperator.reactive.element.Element;
import org.xmloperator.reactive.element.ParentElement;
import org.xmloperator.reactive.element.RWElement;

public class ElementImpl implements Element {

	final String namespaceURI;
	final String localName;
	protected ParentElementImpl parent = null;
	protected ElementImpl previousSibling = null;
	protected ElementImpl nextSibling = null;
	protected RWElement rwElement = null;
	
	public ElementImpl(String namespaceURI, String localName)
			throws BadCharInNameException {
		BadCharInNameException.verifyName(localName);
		this.namespaceURI =
			namespaceURI != null ? namespaceURI.intern() : null;
		this.localName = localName;
	}

	RWElement getRWElement() {
		if (this.rwElement == null)
			this.rwElement = new RWElementImpl(this);
		return this.rwElement;
	}

	////////////////
	// Write methods
	////////////////

	final void insertBefore(ElementImpl element) {
		if (this.parent == null)
			throw new IllegalElementChangeException();
		if (element.parent != null || element == this.root())
			throw new IllegalArgumentException();
		if (this.parent.childrenContraint != null)
			this.parent.childrenContraint.verifyConstraint(
					this.parent, element, true);
		element.parent = this.parent;
		this.parent.childCount += 1;
		if (this == this.parent.firstChild)
			this.parent.firstChild = element;
		element.previousSibling = this.previousSibling;
		element.nextSibling = this;
		if (this.previousSibling != null)
			this.previousSibling.nextSibling = element;
		this.previousSibling = element;
	}
	
	final void remove() {
		if (this.parent == null)
			return;
		if (this.parent.childrenContraint != null)
			this.parent.childrenContraint.verifyConstraint(
					this.parent, this, false);
		this.parent.childCount -= 1;
		if (this.parent.firstChild == this)
			this.parent.firstChild = this.nextSibling;
		if (this.parent.lastChild == this)
			this.parent.lastChild = this.previousSibling;
		if (this.previousSibling != null)
			this.previousSibling.nextSibling =
				this.nextSibling;
		if (this.nextSibling != null)
			this.nextSibling.previousSibling =
				this.previousSibling;
		this.parent = null;
		this.previousSibling = null;
		this.nextSibling = null;
	}
	
	/////////////////////////
	// Element implementation
	/////////////////////////
	
	public boolean isDataElement() {
		return false;
	}

	public boolean isParentElement() {
		return false;
	}
	
	public final String getLocalName() {
		return this.localName;
	}

	public final String getNamespaceURI() {
		return this.namespaceURI;
	}

	public final ParentElement getParent() {
		return this.parent;
	}

	public final Element root() {
		ElementImpl root = this;
		while (root.parent != null)
			root = root.parent;
		return root;
	}
	
	public final Element getPreviousSibling() {
		return this.previousSibling;
	}

	public final Element getNextSibling() {
		return this.nextSibling;
	}
	
	public boolean equals(Element element) {
		if (element == this)
			return true;
		ElementImpl elementI = (ElementImpl)element;
		return this.namespaceURI == elementI.namespaceURI
			&& this.localName.equals(elementI.localName);
	}

	public int previousSiblingCount() {
		int count = 0;
		ElementImpl element = this.previousSibling;
		while (element != null) {
			count += 1;
			element = element.previousSibling;
		}
		return count;
	}

	public String hierarchicalPosition() {
		StringBuffer sb = new StringBuffer();
		sb.append(']');
		ElementImpl element = this;
		while (element.parent != null) {
			sb.insert(0, 1 + element.previousSiblingCount());
			sb.insert(0, '.');
			element = element.parent;
		}
		sb.insert(0, "[");
		return sb.toString();
	}

	////////////////////////
	// Object implementation
	////////////////////////
	
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append('<');
		sb.append(this.localName);
		sb.append('>');
		return sb.toString();
	}
}
