/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.impl;

import org.xmloperator.reactive.Action;
import org.xmloperator.reactive.Event;
import org.xmloperator.reactive.ReactiveEntity;

import org.xmloperator.reactive.exception.ActionAlreadyExecutedException;

/**
 * Action implementation.
 * This Action has no effect at execution.
 */
public class ActionImpl implements Action {
	private final int priority;
	private Event reactionEvent = null;
	private boolean isExecuted = false;
	
	public ActionImpl() {
		this.priority = PRIORITY_MEDIUM;
	}
	
	public ActionImpl(int priority) {
		if (priority < PRIORITY_LOW || priority > PRIORITY_HIGH)
			throw new IllegalArgumentException();
		this.priority = priority;
	}
	
	final void sign(ReactiveEntity entity) {
		if (this.reactionEvent != null)
			throw new IllegalStateException();
		this.reactionEvent = entity.getExclusiveMessageEvent();
	}
	
	////////////////////////
	// Action implementation
	////////////////////////

	public final int getPriority() {
		return this.priority;
	}

	public final Event getReactionEvent() {
		return this.reactionEvent;
	}
	
	public final boolean isExecuted() {
		return this.isExecuted;
	}
	
	public void execute(long permissionKey) {
		if (this.isExecuted)
			throw new ActionAlreadyExecutedException();
		this.isExecuted = true;
	}

	public boolean isTraceable() {
		return true;
	}
}
