/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.impl;

import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;

import org.xmloperator.reactive.Event;
import org.xmloperator.reactive.Message;
import org.xmloperator.reactive.ReactiveCommunity;

/**
 * Event implementation.
 */
class EventImpl implements Event {
	private static AtomicInteger eventCount = new AtomicInteger();
	
	final String id;
	final ReactiveCommunity community;
	final ReactiveEntityImpl uniqueDestinationEntity;
	
	private volatile long generationInstant = 0;
	
	EventImpl(ReactiveCommunity community,
			ReactiveEntityImpl uniqueDestinationEntity) {
		if (community == null)
			throw new IllegalArgumentException();
		{
			StringBuffer sb = new StringBuffer();
			sb.append("E");
			sb.append(EventImpl.eventCount.incrementAndGet());
			if (uniqueDestinationEntity != null) {
				sb.append("[");
				sb.append(uniqueDestinationEntity.getEntityId());
				sb.append("]");
			}
			this.id = sb.toString().intern();
		}
		this.community = community;
		this.uniqueDestinationEntity = uniqueDestinationEntity;
	}

	EventImpl(ReactiveCommunity community) {
		this(community, null);
	}

	/**
	 * Generates this Event at the current instant.
	 * 
	 * @param instant
	 *            the current instant.
	 * @return <tt>true</tt> if this is the first generation during this
	 *         instant, <tt>false</tt> elsewhere.
	 */
	final boolean generate(long instant) {
		if (this.generationInstant > instant)
			throw new IllegalStateException();
		synchronized (this) {
			if (this.generationInstant != instant) {
				this.generationInstant = instant;
				return true;
			}
		}
		return false;
	}
	
	///////////////////////
	// Event implementation
	///////////////////////
	
	public final ReactiveCommunity getCommunity() {
		return this.community;
	}

	public final boolean isExclusive() {
		return this.uniqueDestinationEntity != null;
	}
	
	public final String getUniqueDestinationEntityId() {
		return this.uniqueDestinationEntity != null
			? this.uniqueDestinationEntity.getEntityId()
			: null;
	}
	
	public List<Message> getMessageList() {
		return null;
	}

	////////////////////////
	// Object implementation
	////////////////////////

	public String toString() {
		return this.id;
	}
}
